#pragma once

#include <array>

namespace mbgl {

using vec2 = std::array<double, 2>;
using vec3 = std::array<double, 3>;
using vec3f = std::array<float, 3>;
using vec3i = std::array<int, 3>;
using vec4 = std::array<double, 4>;

} // namespace mbgl